// Run the code when the DOM is ready
$( pieChart );

function pieChart() {

  // Config settings
  var chartSizePercent = 100;                        // 图表半径相对于画布的宽度/高度(百分比)
  var sliceBorderWidth = 0.1;                         // 每一片边缘的边界宽度（像素）
  var sliceBorderStyle = "#fff";                    // 每一片边缘的边界颜色（color）
  var sliceGradientColour = "#DDD";                 // 图梯度的一个颜色的使用
  var maxPullOutDistance =15;                      // 在像素点上，当点击点击时，将切片拉出距离
  var pullOutFrameStep = 1;                         // 每一个动画帧的移动一个切片有多少像素
  var pullOutFrameInterval = 30;                    // 每一个动画帧之间的时间（毫秒）
  var pullOutLabelPadding = 10;                     // 拉出切片和标签之间的填充
  var pullOutLabelFont = "bold 0.875em 'Trebuchet MS', Verdana, sans-serif";  // 拉出螺旋label的字体
  var pullOutValueFont = "bold 0.875em 'Trebuchet MS', Verdana, sans-serif";  // 拔出切片值字体
  var pullOutValuePrefix = "$";                     // 拔出切片值前缀
  var pullOutShadowColour = "rgba( 0, 0, 0, 0.1 )";  // 颜色使用拔片的影子
  var pullOutShadowOffsetX = 5;                     // X轴方向偏移（像素）的拔出片的影子
  var pullOutShadowOffsetY = 5;                     // Y-轴方向偏移（像素）的拔出片的影子
  var pullOutShadowBlur = 5;                        // 多少模糊的影子拉片
  var pullOutBorderWidth = 1;                       // 宽度（像素）的拔片边界
  var pullOutBorderStyle = "#999";                  // 拔出片的边框颜色
  var chartStartAngle = -.5 * Math.PI;              // 十二点开始，而不是三点

  // 声明一些变量的图表
  var canvas;                       // 页面中的画布元素
  var currentPullOutSlice = -1;     // 该片目前拉出（- 1 =无片）
  var currentPullOutDistance = 0;   // 在动画中，拉出了多少像素的像素
  var animationId = 0;              //轨道由setinterval()动画间隔的ID
  var chartData = [];               // 图表数据（标签、值和角度）
  var chartColours = [];            // 图的颜色（从HTML表格拉）
  var totalValue = 0;               // 图表中的所有值
  var canvasWidth;                  // 画布的宽度，以像素为单位
  var canvasHeight;                 // 画布的高度，以像素为单位
  var centreX;                      // X-帆布/图表中心的坐标
  var centreY;                      // Y-帆布/图表中心的坐标
  var chartRadius;                  // 半径的饼图，在像素

  // 设置和绘制图表
  init();


  /**
   * 设置图表数据和颜色，以及图表和表格点击处理程序，
   * 和绘制初始饼图
   */

  function init() {

    // 获取页面中的画布元素
  var intse=  document.getElementsByClassName('chart');
    canvas =intse[0];


    // 退出，如果浏览器是不是帆布能力
    if ( typeof canvas.getContext === 'undefined' ) return;

    // 初始化的帆布和图的一些性质
    canvasWidth = canvas.width;
    canvasHeight = canvas.height;
    centreX = canvasWidth / 2;
    centreY = canvasHeight / 2;
    chartRadius = Math.min( canvasWidth, canvasHeight ) / 2 * ( chartSizePercent / 100 );

    // 从表中抓取数据，
    // 并将点击处理程序分配给表数据单元

    var currentRow = -1;
    var currentCell = 0;

    $('.chartData td').each( function() {
      currentCell++;
      if ( currentCell % 2 != 0 ) {
        currentRow++;
        chartData[currentRow] = [];
        chartData[currentRow]['label'] = $(this).text();
      } else {
       var value = parseFloat($(this).text());
       totalValue += value;
       value = value.toFixed(2);
       chartData[currentRow]['value'] = value;
      }

      // 存储这个单元格中的切片索引，并将其附加到它
      $(this).data( 'slice', currentRow );
      $(this).click( handleTableClick );

      // 提取和储存细胞的颜色
      if ( rgb = $(this).css('color').match( /rgb\((\d+), (\d+), (\d+)/) ) {
        chartColours[currentRow] = [ rgb[1], rgb[2], rgb[3] ];
      } else if ( hex = $(this).css('color').match(/#([a-fA-F0-9]{2})([a-fA-F0-9]{2})([a-fA-F0-9]{2})/) ) {
        chartColours[currentRow] = [ parseInt(hex[1],16) ,parseInt(hex[2],16), parseInt(hex[3], 16) ];
      } else {
        alert( "错误：颜色无法确定！请使用格式的# xxxxxx指定表的颜色”" );
        return;
      }

    } );

    // 现在计算并存储在图表数据中的每一个切片的开始和结束角度

    var currentPos = 0; // 在圆饼（从0到1）的当前位置

    for ( var slice in chartData ) {
      chartData[slice]['startAngle'] = 2 * Math.PI * currentPos;
      chartData[slice]['endAngle'] = 2 * Math.PI * ( currentPos + ( chartData[slice]['value'] / totalValue ) );
      currentPos += chartData[slice]['value'] / totalValue;
    }

    // All ready! Now draw the pie chart, and add the click handler to it
    drawChart();
    $('#chart').click ( handleChartClick );
  }





  /**
   * Process mouse clicks in the table area.
   *
   * Retrieve the slice number from the jQuery data stored in the
   * clicked table cell, then toggle the slice
   *
   * @param Event The click event
   */

  function handleTableClick ( clickEvent ) {
    var slice = $(this).data('slice');
    toggleSlice ( slice );
  }


  /**
   * Push a slice in or out.
   *
   * If it's already pulled out, push it in. Otherwise, pull it out.
   *
   * @param Number The slice index (between 0 and the number of slices - 1)
   */

  function toggleSlice ( slice ) {
    if ( slice == currentPullOutSlice ) {
      pushIn();
    } else {
      startPullOut ( slice );
    }
  }


  /**点击文字弹出事件
   * Start pulling a slice out from the pie.
   *
   * @param Number The slice index (between 0 and the number of slices - 1)
   *  function startPullOut ( slice ) {

    // Exit if we're already pulling out this slice
    if ( currentPullOutSlice == slice ) return;

    // Record the slice that we're pulling out, clear any previous animation, then start the animation
    currentPullOutSlice = slice;
    currentPullOutDistance = 0;
    clearInterval( animationId );
    animationId = setInterval( function() { animatePullOut( slice ); }, pullOutFrameInterval );

    // Highlight the corresponding row in the key table
    $('#chartData td').removeClass('highlight');
    var labelCell = $('#chartData td:eq(' + (slice*2) + ')');
    var valueCell = $('#chartData td:eq(' + (slice*2+1) + ')');
    labelCell.addClass('highlight');
    valueCell.addClass('highlight');
  }
   */




  /**
   * Draw a frame of the pull-out animation.
   *
   * @param Number The index of the slice being pulled out
   */

  function animatePullOut ( slice ) {

    // Pull the slice out some more
    currentPullOutDistance += pullOutFrameStep;

    // If we've pulled it right out, stop animating
    if ( currentPullOutDistance >= maxPullOutDistance ) {
      clearInterval( animationId );
      return;
    }

    // Draw the frame
    drawChart();
  }


  /**
   * Push any pulled-out slice back in.
   *
   * Resets the animation variables and redraws the chart.
   * Also un-highlights all rows in the table.
   */

  function pushIn() {
    currentPullOutSlice = -1;
    currentPullOutDistance = 0;
    clearInterval( animationId );
    drawChart();
    $('.chartData td').removeClass('highlight');
  }


  /**
   * Draw the chart.
   *
   * Loop through each slice of the pie, and draw it.
   */

  function drawChart() {

    // Get a drawing context
    var context = canvas.getContext('2d');

    // Clear the canvas, ready for the new frame
    context.clearRect ( 0, 0, canvasWidth, canvasHeight );

    // Draw each slice of the chart, skipping the pull-out slice (if any)
    for ( var slice in chartData ) {
      if ( slice != currentPullOutSlice ) drawSlice( context, slice );
    }

    // If there's a pull-out slice in effect, draw it.
    // (We draw the pull-out slice last so its drop shadow doesn't get painted over.)
    if ( currentPullOutSlice != -1 ) drawSlice( context, currentPullOutSlice );
  }


  /**
   * 在图表中画一个单独的切片。
   *
   * @param 上下文的画布上下文来绘制
   * @param 将切片的索引编号为
   */

  function drawSlice ( context, slice ) {

    // 计算切片的调整开始和结束角度
    var startAngle = chartData[slice]['startAngle']  + chartStartAngle;
    var endAngle = chartData[slice]['endAngle']  + chartStartAngle;

    if ( slice == currentPullOutSlice ) {

      // 我们正在拉（或已经把）这片。
      // 从圆饼中心的偏移量，绘制文本标签，
      // 添加一个阴影.

      var midAngle = (startAngle + endAngle) / 2;
      var actualPullOutDistance = currentPullOutDistance * easeOut( currentPullOutDistance/maxPullOutDistance, .8 );
      startX = centreX + Math.cos(midAngle) * actualPullOutDistance;
      startY = centreY + Math.sin(midAngle) * actualPullOutDistance;
      context.fillStyle = 'rgb(' + chartColours[slice].join(',') + ')';
      context.textAlign = "center";
      context.font = pullOutLabelFont;
      context.fillText( chartData[slice]['label'], centreX + Math.cos(midAngle) * ( chartRadius + maxPullOutDistance + pullOutLabelPadding ), centreY + Math.sin(midAngle) * ( chartRadius + maxPullOutDistance + pullOutLabelPadding ) );
      context.font = pullOutValueFont;
      context.fillText( pullOutValuePrefix + chartData[slice]['value'] + " (" + ( parseInt( chartData[slice]['value'] / totalValue * 100 + .5 ) ) +  "%)", centreX + Math.cos(midAngle) * ( chartRadius + maxPullOutDistance + pullOutLabelPadding ), centreY + Math.sin(midAngle) * ( chartRadius + maxPullOutDistance + pullOutLabelPadding ) + 20 );
      context.shadowOffsetX = pullOutShadowOffsetX;
      context.shadowOffsetY = pullOutShadowOffsetY;
      context.shadowBlur = pullOutShadowBlur;

    } else {

      // 这片没有被拔出，所以从圆饼中心汲取它
      startX = centreX;
      startY = centreY;
    }

    // 设置渐变填充为切片
    var sliceGradient = context.createLinearGradient( 0, 0, canvasWidth*.100, canvasHeight*.100);
    sliceGradient.addColorStop( 0, sliceGradientColour );
    sliceGradient.addColorStop( 1, 'rgb(' + chartColours[slice].join(',') + ')' );

    // Draw the slice
    context.beginPath();
    context.moveTo( startX, startY );
    context.arc( startX, startY, chartRadius, startAngle, endAngle, false );
    context.lineTo( startX, startY );
    context.closePath();
    context.fillStyle = sliceGradient;
    context.shadowColor = ( slice == currentPullOutSlice ) ? pullOutShadowColour : "rgba( 0, 0, 0, 0 )";
    context.fill();
    context.shadowColor = "rgba( 0, 0, 0, 0 )";

    // Style the slice border appropriately
    if ( slice == currentPullOutSlice ) {
      context.lineWidth = pullOutBorderWidth;
      context.strokeStyle = pullOutBorderStyle;
    } else {
      context.lineWidth = sliceBorderWidth;
      context.strokeStyle = sliceBorderStyle;
    }

    // Draw the slice border
    context.stroke();
  }




};